@@ -2,7 +2,8 @@
 #'@title  Data-Driven Nonparametric Shape Restriction and Parametric Model Specification Testing using Binscatter
 #'@description \code{binstest} implements binscatter-based hypothesis testing procedures for parametric functional
 #'             forms of and nonparametric shape restrictions on the regression function of interest, following the results
-#'             in \href{https://arxiv.org/abs/1902.09608}{Cattaneo, Crump, Farrell and Feng (2022a)}.
+#'             in \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_AER.pdf}{Cattaneo, Crump, Farrell and Feng (2023a)} and
+#'             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_NonlinearBinscatter.pdf}{Cattaneo, Crump, Farrell and Feng (2023b)}.
 #'             If the binning scheme is not set by the user,
 #'             the companion function \code{\link{binsregselect}} is used to implement binscatter in a
 #'             data-driven way and inference procedures are based on robust bias correction.
@@ -24,8 +25,8 @@
 #'@param  nolink if true, the function within the inverse link function is reported instead of the conditional mean function for the outcome.
 #'@param  testmodel a vector or a logical value. It sets the degree of polynomial and the number of smoothness constraints for parametric model specification
 #'                  testing. If \code{testmodel=c(p,s)} is specified, a piecewise polynomial of degree \code{p} with \code{s} smoothness constraints is used.
-#'                  If \code{testmodel=T} or \code{testmodel=NULL} (default) is specified, \code{testmodel=c(1,1)} is used unless the degree \code{p} and the smoothness \code{s}
-#'                  selection is requested via the option \code{pselect} (see more details in the explanation of \code{pselect}).
+#'                  If \code{testmodel=T} or \code{testmodel=NULL} (default) is specified, \code{testmodel=c(1,1)} is used unless the degree \code{p} or the smoothness \code{s}
+#'                  selection is requested via the option \code{pselect} or \code{sselect} (see more details in the explanation of \code{pselect} and \code{sselect}).
 #'@param  testmodelparfit a data frame or matrix which contains the evaluation grid and fitted values of the model(s) to be
 #'                        tested against.  The column contains a series of evaluation points
 #'                        at which the binscatter model and the parametric model of interest are compared with
@@ -34,8 +35,8 @@
 #'@param  testmodelpoly degree of a global polynomial model to be tested against.
 #'@param  testshape a vector or a logical value. It sets the degree of polynomial and the number of smoothness constraints for nonparametric shape restriction
 #'                  testing. If \code{testshape=c(p,s)} is specified, a piecewise polynomial of degree \code{p} with \code{s} smoothness constraints is used.
-#'                  If \code{testshape=T} or \code{testshape=NULL} (default) is specified, \code{testshape=c(1,1)} is used unless the degree \code{p} and smoothness \code{s} selection
-#'                  is requested via the option \code{pselect} (see more details in the explanation of \code{pselect}).
+#'                  If \code{testshape=T} or \code{testshape=NULL} (default) is specified, \code{testshape=c(1,1)} is used unless the degree \code{p} or smoothness \code{s} selection
+#'                  is requested via the option \code{pselect} or \code{sselect} (see more details in the explanation of \code{pselect} and \code{sselect}).
 #'@param  testshapel a vector of null boundary values for hypothesis testing. Each number \code{a} in the vector
 #'                   corresponds to one boundary of a one-sided hypothesis test to the left of the form
 #'                   \code{H0: sup_x mu(x)<=a}.
@@ -73,16 +74,16 @@
 #'@param  nbinsrot initial number of bins value used to construct the DPI number of bins selector.
 #'                 If not specified, the data-driven ROT selector is used instead.
 #'@param  randcut upper bound on a uniformly distributed variable used to draw a subsample for bins/degree/smoothness selection.
-#'                Observations for which \code{runif()<=#} are used. # must be between 0 and 1. By default, \code{max(5,000, 0.01n)} observations
-#'                are used if the samples size \code{n>5,000}.
+#'                Observations for which \code{runif()<=#} are used. # must be between 0 and 1. By default, \code{max(5000, 0.01n)} observations
+#'                are used if the samples size \code{n>5000}.
 #'@param  nsims number of random draws for hypothesis testing. The default is
 #'              \code{nsims=500}, which corresponds to 500 draws from a standard Gaussian random vector of size
-#'              \code{[(p+1)*J - (J-1)*s]}. A larger number of draws is recommended to obtain the final results.
+#'              \code{[(p+1)*J - (J-1)*s]}. Setting at least \code{nsims=2000} is recommended to obtain the final results.
 #'@param  simsgrid number of evaluation points of an evenly-spaced grid within each bin used for evaluation of
 #'                 the supremum (infimum or Lp metric) operation needed to construct hypothesis testing
 #'                 procedures. The default is \code{simsgrid=20}, which corresponds to 20 evenly-spaced
 #'                 evaluation points within each bin for approximating the supremum (infimum or Lp metric) operator.
-#'                 A larger number of evaluation points is recommended to obtain the final results.
+#'                 Setting at least \code{simsgrid=50} is recommended to obtain the final results.
 #'@param  simsseed seed for simulation.
 #'@param  vce procedure to compute the variance-covariance matrix estimator. For least squares regression and generalized linear regression, the allowed options are the same as that for \code{\link{binsreg}} or \code{\link{binsqreg}}.
 #'            For quantile regression, the allowed options are the same as that for \code{\link{binsqreg}}.
@@ -92,7 +93,7 @@
 #'@param  dfcheck adjustments for minimum effective sample size checks, which take into account number of unique
 #'                values of \code{x} (i.e., number of mass points), number of clusters, and degrees of freedom of
 #'                the different stat models considered. The default is \code{dfcheck=c(20, 30)}.
-#'                See \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2022_Stata.pdf}{Cattaneo, Crump, Farrell and Feng (2022b)} for more details.
+#'                See \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_Stata.pdf}{Cattaneo, Crump, Farrell and Feng (2023c)} for more details.
 #'@param  masspoints how mass points in \code{x} are handled. Available options:
 #'                   \itemize{
 #'                   \item \code{"on"} all mass point and degrees of freedom checks are implemented. Default.
@@ -132,14 +133,16 @@
 #'
 #' Richard K. Crump, Federal Reserve Bank of New York, New York, NY. \email{richard.crump@ny.frb.org}.
 #'
-#' Max H. Farrell, University of Chicago, Chicago, IL. \email{max.farrell@chicagobooth.edu}.
+#' Max H. Farrell, UC Santa Barbara, Santa Barbara, CA. \email{mhfarrell@gmail.com}.
 #'
 #' Yingjie Feng (maintainer), Tsinghua University, Beijing, China. \email{fengyingjiepku@gmail.com}.
 #'
 #'@references
-#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2022a: \href{https://arxiv.org/abs/1902.09608}{On Binscatter}. Working Paper.
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023a: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_AER.pdf}{On Binscatter}. Working Paper.
 #'
-#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2022b: \href{https://arxiv.org/abs/1902.09615}{Binscatter Regressions}. Working Paper.
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023b: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_NonlinearBinscatter.pdf}{Nonlinear Binscatter Methods}. Working Paper.
+#'
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023c: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_Stata.pdf}{Binscatter Regressions}. Working Paper.
 #'
 #'@seealso \code{\link{binsreg}}, \code{\link{binsqreg}}, \code{\link{binsglm}}, \code{\link{binsregselect}}.
 #'
@@ -477,7 +480,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
   if (exit>0) stop()
 
   if (nsims<2000|simsgrid<50) {
-    print("Note: A large number of random draws/evaluation points is recommended to obtain the final results.")
+    print("Note: Setting at least nsims=2000 and simsgrid=50 is recommended to obtain the final results.")
   }
 
   #################################################
@@ -533,7 +536,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
     }
   }
   if (selection=="U") {
-    warning("Testing procedures are valid when nbins is much larger than the IMSE-optimal choice.")
+    warning("Testing procedures are valid when nbins is much larger than the IMSE-optimal choice. Compare your choice with the IMSE-optimal one obtained by binsregselect().")
   }
 
   nL <- length(testshapel); nR <- length(testshaper); nT <- length(testshape2)
@@ -592,7 +595,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
     randcut1k <- randcut
     if (is.null(randcut) & N>5000) {
       randcut1k <- max(5000/N, 0.01)
-      warning("To speed up computation, bin/degree selection uses a subsample of roughly max(5,000, 0.01n) observations if the sample size n>5,000. To use the full sample, set randcut=1.")
+      warning("To speed up computation, bin/degree selection uses a subsample of roughly max(5000, 0.01n) observations if the sample size n>5000. To use the full sample, set randcut=1.")
     }
     if (selection=="J") {
       binselect <- binsregselect(y, x, w, deriv=deriv,
